@@ -15,109 +15,81 @@
 #    under the License.
 
 import functools
-import threading
 
-from oslo.utils import reflection
-from oslo.utils import timeutils
+from oslo_utils import timeutils
 
 from taskflow.engines.action_engine import executor
-from taskflow.engines.worker_based import cache
 from taskflow.engines.worker_based import protocol as pr
 from taskflow.engines.worker_based import proxy
+from taskflow.engines.worker_based import types as wt
 from taskflow import exceptions as exc
 from taskflow import logging
 from taskflow import task as task_atom
-from taskflow.types import timing as tt
+from taskflow.types import periodic
+from taskflow.utils import kombu_utils as ku
 from taskflow.utils import misc
 from taskflow.utils import threading_utils as tu
 
 LOG = logging.getLogger(__name__)
 
 
-class PeriodicWorker(object):
-    """Calls a set of functions when activated periodically.
-
-    NOTE(harlowja): the provided timeout object determines the periodicity.
-    """
-    def __init__(self, timeout, functors):
-        self._timeout = timeout
-        self._functors = []
-        for f in functors:
-            self._functors.append((f, reflection.get_callable_name(f)))
-
-    def start(self):
-        while not self._timeout.is_stopped():
-            for (f, f_name) in self._functors:
-                LOG.debug("Calling periodic function '%s'", f_name)
-                try:
-                    f()
-                except Exception:
-                    LOG.warn("Failed to call periodic function '%s'", f_name,
-                             exc_info=True)
-            self._timeout.wait()
-
-    def stop(self):
-        self._timeout.interrupt()
-
-    def reset(self):
-        self._timeout.reset()
-
-
 class WorkerTaskExecutor(executor.TaskExecutor):
     """Executes tasks on remote workers."""
 
     def __init__(self, uuid, exchange, topics,
-                 transition_timeout=pr.REQUEST_TIMEOUT, **kwargs):
+                 transition_timeout=pr.REQUEST_TIMEOUT,
+                 url=None, transport=None, transport_options=None,
+                 retry_options=None):
         self._uuid = uuid
-        self._topics = topics
-        self._requests_cache = cache.RequestsCache()
+        self._requests_cache = wt.RequestsCache()
         self._transition_timeout = transition_timeout
-        self._workers_cache = cache.WorkersCache()
-        self._workers_arrival = threading.Condition()
-        handlers = {
-            pr.NOTIFY: [
-                self._process_notify,
-                functools.partial(pr.Notify.validate, response=True),
-            ],
+        type_handlers = {
             pr.RESPONSE: [
                 self._process_response,
                 pr.Response.validate,
             ],
         }
-        self._proxy = proxy.Proxy(uuid, exchange, handlers,
-                                  self._on_wait, **kwargs)
-        self._proxy_thread = None
-        self._periodic = PeriodicWorker(tt.Timeout(pr.NOTIFY_PERIOD),
-                                        [self._notify_topics])
-        self._periodic_thread = None
+        self._proxy = proxy.Proxy(uuid, exchange,
+                                  type_handlers=type_handlers,
+                                  on_wait=self._on_wait, url=url,
+                                  transport=transport,
+                                  transport_options=transport_options,
+                                  retry_options=retry_options)
+        # NOTE(harlowja): This is the most simplest finder impl. that
+        # doesn't have external dependencies (outside of what this engine
+        # already requires); it though does create periodic 'polling' traffic
+        # to workers to 'learn' of the tasks they can perform (and requires
+        # pre-existing knowledge of the topics those workers are on to gather
+        # and update this information).
+        self._finder = wt.ProxyWorkerFinder(uuid, self._proxy, topics)
+        self._finder.on_worker = self._on_worker
+        self._helpers = tu.ThreadBundle()
+        self._helpers.bind(lambda: tu.daemon_thread(self._proxy.start),
+                           after_start=lambda t: self._proxy.wait(),
+                           before_join=lambda t: self._proxy.stop())
+        p_worker = periodic.PeriodicWorker.create([self._finder])
+        if p_worker:
+            self._helpers.bind(lambda: tu.daemon_thread(p_worker.start),
+                               before_join=lambda t: p_worker.stop(),
+                               after_join=lambda t: p_worker.reset(),
+                               before_start=lambda t: p_worker.reset())
 
-    def _process_notify(self, notify, message):
-        """Process notify message from remote side."""
-        LOG.debug("Started processing notify message '%s'",
-                  message.delivery_tag)
-        topic = notify['topic']
-        tasks = notify['tasks']
-
-        # Add worker info to the cache
-        LOG.debug("Received that tasks %s can be processed by topic '%s'",
-                  tasks, topic)
-        with self._workers_arrival:
-            self._workers_cache[topic] = tasks
-            self._workers_arrival.notify_all()
-
-        # Publish waiting requests
-        for request in self._requests_cache.get_waiting_requests(tasks):
+    def _on_worker(self, worker):
+        """Process new worker that has arrived (and fire off any work)."""
+        for request in self._requests_cache.get_waiting_requests(worker):
             if request.transition_and_log_error(pr.PENDING, logger=LOG):
-                self._publish_request(request, topic)
+                self._publish_request(request, worker)
 
     def _process_response(self, response, message):
         """Process response from remote side."""
         LOG.debug("Started processing response message '%s'",
-                  message.delivery_tag)
+                  ku.DelayedPretty(message))
         try:
             task_uuid = message.properties['correlation_id']
         except KeyError:
-            LOG.warning("The 'correlation_id' message property is missing")
+            LOG.warning("The 'correlation_id' message property is"
+                        " missing in message '%s'",
+                        ku.DelayedPretty(message))
         else:
             request = self._requests_cache.get(task_uuid)
             if request is not None:
@@ -142,7 +114,7 @@ class WorkerTaskExecutor(executor.TaskExecutor):
                         del self._requests_cache[request.uuid]
                         request.set_result(**response.data)
                 else:
-                    LOG.warning("Unexpected response status: '%s'",
+                    LOG.warning("Unexpected response status '%s'",
                                 response.state)
             else:
                 LOG.debug("Request with id='%s' not found", task_uuid)
@@ -191,16 +163,16 @@ class WorkerTaskExecutor(executor.TaskExecutor):
                                         progress_callback)
             request.result.add_done_callback(lambda fut: cleaner())
 
-        # Get task's topic and publish request if topic was found.
-        topic = self._workers_cache.get_topic_by_task(request.task_cls)
-        if topic is not None:
+        # Get task's worker and publish request if worker was found.
+        worker = self._finder.get_worker_for_task(task)
+        if worker is not None:
             # NOTE(skudriashev): Make sure request is set to the PENDING state
             # before putting it into the requests cache to prevent the notify
             # processing thread get list of waiting requests and publish it
             # before it is published here, so it wouldn't be published twice.
             if request.transition_and_log_error(pr.PENDING, logger=LOG):
                 self._requests_cache[request.uuid] = request
-                self._publish_request(request, topic)
+                self._publish_request(request, worker)
         else:
             LOG.debug("Delaying submission of '%s', no currently known"
                       " worker/s available to process it", request)
@@ -208,14 +180,14 @@ class WorkerTaskExecutor(executor.TaskExecutor):
 
         return request.result
 
-    def _publish_request(self, request, topic):
+    def _publish_request(self, request, worker):
         """Publish request to a given topic."""
-        LOG.debug("Submitting execution of '%s' to topic '%s' (expecting"
+        LOG.debug("Submitting execution of '%s' to worker '%s' (expecting"
                   " response identified by reply_to=%s and"
-                  " correlation_id=%s)", request, topic, self._uuid,
+                  " correlation_id=%s)", request, worker, self._uuid,
                   request.uuid)
         try:
-            self._proxy.publish(request, topic,
+            self._proxy.publish(request, worker.topic,
                                 reply_to=self._uuid,
                                 correlation_id=request.uuid)
         except Exception:
@@ -226,10 +198,6 @@ class WorkerTaskExecutor(executor.TaskExecutor):
                     del self._requests_cache[request.uuid]
                     request.set_result(failure)
 
-    def _notify_topics(self):
-        """Cyclically called to publish notify message to each topic."""
-        self._proxy.publish(pr.Notify(), self._topics, reply_to=self._uuid)
-
     def execute_task(self, task, task_uuid, arguments,
                      progress_callback=None):
         return self._submit_task(task, task_uuid, pr.EXECUTE, arguments,
@@ -250,39 +218,15 @@ class WorkerTaskExecutor(executor.TaskExecutor):
         return how many workers are still needed, otherwise it will
         return zero.
         """
-        if workers <= 0:
-            raise ValueError("Worker amount must be greater than zero")
-        w = None
-        if timeout is not None:
-            w = tt.StopWatch(timeout).start()
-        with self._workers_arrival:
-            while len(self._workers_cache) < workers:
-                if w is not None and w.expired():
-                    return workers - len(self._workers_cache)
-                timeout = None
-                if w is not None:
-                    timeout = w.leftover()
-                self._workers_arrival.wait(timeout)
-            return 0
+        return self._finder.wait_for_workers(workers=workers,
+                                             timeout=timeout)
 
     def start(self):
         """Starts proxy thread and associated topic notification thread."""
-        if not tu.is_alive(self._proxy_thread):
-            self._proxy_thread = tu.daemon_thread(self._proxy.start)
-            self._proxy_thread.start()
-            self._proxy.wait()
-        if not tu.is_alive(self._periodic_thread):
-            self._periodic.reset()
-            self._periodic_thread = tu.daemon_thread(self._periodic.start)
-            self._periodic_thread.start()
+        self._helpers.start()
 
     def stop(self):
         """Stops proxy thread and associated topic notification thread."""
-        if self._periodic_thread is not None:
-            self._periodic.stop()
-            self._periodic_thread.join()
-            self._periodic_thread = None
-        if self._proxy_thread is not None:
-            self._proxy.stop()
-            self._proxy_thread.join()
-            self._proxy_thread = None
+        self._helpers.stop()
+        self._requests_cache.clear(self._handle_expired_request)
+        self._finder.clear()
