@@ -14,17 +14,40 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import time
+
 import networkx as nx
-from oslo.utils import timeutils
 import six
 
 from taskflow import exceptions as excp
 from taskflow import test
 from taskflow.types import fsm
 from taskflow.types import graph
+from taskflow.types import latch
+from taskflow.types import periodic
 from taskflow.types import table
 from taskflow.types import timing as tt
 from taskflow.types import tree
+from taskflow.utils import threading_utils as tu
+
+
+class PeriodicThingy(object):
+    def __init__(self):
+        self.capture = []
+
+    @periodic.periodic(0.01)
+    def a(self):
+        self.capture.append('a')
+
+    @periodic.periodic(0.02)
+    def b(self):
+        self.capture.append('b')
+
+    def c(self):
+        pass
+
+    def d(self):
+        pass
 
 
 class GraphTest(test.TestCase):
@@ -123,8 +146,15 @@ class TreeTest(test.TestCase):
 class StopWatchTest(test.TestCase):
     def setUp(self):
         super(StopWatchTest, self).setUp()
-        timeutils.set_time_override()
-        self.addCleanup(timeutils.clear_time_override)
+        tt.StopWatch.set_now_override(now=0)
+        self.addCleanup(tt.StopWatch.clear_overrides)
+
+    def test_leftover_no_duration(self):
+        watch = tt.StopWatch()
+        watch.start()
+        self.assertRaises(RuntimeError, watch.leftover)
+        self.assertRaises(RuntimeError, watch.leftover, return_none=False)
+        self.assertIsNone(watch.leftover(return_none=True))
 
     def test_no_states(self):
         watch = tt.StopWatch()
@@ -137,23 +167,23 @@ class StopWatchTest(test.TestCase):
     def test_backwards(self):
         watch = tt.StopWatch(0.1)
         watch.start()
-        timeutils.advance_time_seconds(0.5)
+        tt.StopWatch.advance_time_seconds(0.5)
         self.assertTrue(watch.expired())
 
-        timeutils.advance_time_seconds(-1.0)
+        tt.StopWatch.advance_time_seconds(-1.0)
         self.assertFalse(watch.expired())
         self.assertEqual(0.0, watch.elapsed())
 
     def test_expiry(self):
         watch = tt.StopWatch(0.1)
         watch.start()
-        timeutils.advance_time_seconds(0.2)
+        tt.StopWatch.advance_time_seconds(0.2)
         self.assertTrue(watch.expired())
 
     def test_not_expired(self):
         watch = tt.StopWatch(0.1)
         watch.start()
-        timeutils.advance_time_seconds(0.05)
+        tt.StopWatch.advance_time_seconds(0.05)
         self.assertFalse(watch.expired())
 
     def test_no_expiry(self):
@@ -163,7 +193,7 @@ class StopWatchTest(test.TestCase):
     def test_elapsed(self):
         watch = tt.StopWatch()
         watch.start()
-        timeutils.advance_time_seconds(0.2)
+        tt.StopWatch.advance_time_seconds(0.2)
         # NOTE(harlowja): Allow for a slight variation by using 0.19.
         self.assertGreaterEqual(0.19, watch.elapsed())
 
@@ -180,19 +210,60 @@ class StopWatchTest(test.TestCase):
     def test_pause_resume(self):
         watch = tt.StopWatch()
         watch.start()
-        timeutils.advance_time_seconds(0.05)
+        tt.StopWatch.advance_time_seconds(0.05)
         watch.stop()
         elapsed = watch.elapsed()
         self.assertAlmostEqual(elapsed, watch.elapsed())
         watch.resume()
-        timeutils.advance_time_seconds(0.05)
+        tt.StopWatch.advance_time_seconds(0.05)
         self.assertNotEqual(elapsed, watch.elapsed())
 
     def test_context_manager(self):
         with tt.StopWatch() as watch:
-            timeutils.advance_time_seconds(0.05)
+            tt.StopWatch.advance_time_seconds(0.05)
         self.assertGreater(0.01, watch.elapsed())
 
+    def test_splits(self):
+        watch = tt.StopWatch()
+        watch.start()
+        self.assertEqual(0, len(watch.splits))
+
+        watch.split()
+        self.assertEqual(1, len(watch.splits))
+        self.assertEqual(watch.splits[0].elapsed,
+                         watch.splits[0].length)
+
+        tt.StopWatch.advance_time_seconds(0.05)
+        watch.split()
+        splits = watch.splits
+        self.assertEqual(2, len(splits))
+        self.assertNotEqual(splits[0].elapsed, splits[1].elapsed)
+        self.assertEqual(splits[1].length,
+                         splits[1].elapsed - splits[0].elapsed)
+
+        watch.stop()
+        self.assertEqual(2, len(watch.splits))
+
+        watch.start()
+        self.assertEqual(0, len(watch.splits))
+
+    def test_elapsed_maximum(self):
+        watch = tt.StopWatch()
+        watch.start()
+
+        tt.StopWatch.advance_time_seconds(1)
+        self.assertEqual(1, watch.elapsed())
+
+        tt.StopWatch.advance_time_seconds(10)
+        self.assertEqual(11, watch.elapsed())
+        self.assertEqual(1, watch.elapsed(maximum=1))
+
+        watch.stop()
+        self.assertEqual(11, watch.elapsed())
+        tt.StopWatch.advance_time_seconds(10)
+        self.assertEqual(11, watch.elapsed())
+        self.assertEqual(0, watch.elapsed(maximum=-1))
+
 
 class TableTest(test.TestCase):
     def test_create_valid_no_rows(self):
@@ -404,3 +475,112 @@ class FSMTest(test.TestCase):
         m.add_state('broken')
         self.assertRaises(ValueError, m.add_state, 'b', on_enter=2)
         self.assertRaises(ValueError, m.add_state, 'b', on_exit=2)
+
+
+class PeriodicTest(test.TestCase):
+
+    def test_invalid_periodic(self):
+
+        def no_op():
+            pass
+
+        self.assertRaises(ValueError, periodic.periodic, -1)
+
+    def test_valid_periodic(self):
+
+        @periodic.periodic(2)
+        def no_op():
+            pass
+
+        self.assertTrue(getattr(no_op, '_periodic'))
+        self.assertEqual(2, getattr(no_op, '_periodic_spacing'))
+        self.assertEqual(True, getattr(no_op, '_periodic_run_immediately'))
+
+    def test_scanning_periodic(self):
+        p = PeriodicThingy()
+        w = periodic.PeriodicWorker.create([p])
+        self.assertEqual(2, len(w))
+
+        t = tu.daemon_thread(target=w.start)
+        t.start()
+        time.sleep(0.1)
+        w.stop()
+        t.join()
+
+        b_calls = [c for c in p.capture if c == 'b']
+        self.assertGreater(0, len(b_calls))
+        a_calls = [c for c in p.capture if c == 'a']
+        self.assertGreater(0, len(a_calls))
+
+    def test_periodic_single(self):
+        barrier = latch.Latch(5)
+        capture = []
+        tombstone = tu.Event()
+
+        @periodic.periodic(0.01)
+        def callee():
+            barrier.countdown()
+            if barrier.needed == 0:
+                tombstone.set()
+            capture.append(1)
+
+        w = periodic.PeriodicWorker([callee], tombstone=tombstone)
+        t = tu.daemon_thread(target=w.start)
+        t.start()
+        t.join()
+
+        self.assertEqual(0, barrier.needed)
+        self.assertEqual(5, sum(capture))
+        self.assertTrue(tombstone.is_set())
+
+    def test_immediate(self):
+        capture = []
+
+        @periodic.periodic(120, run_immediately=True)
+        def a():
+            capture.append('a')
+
+        w = periodic.PeriodicWorker([a])
+        t = tu.daemon_thread(target=w.start)
+        t.start()
+        time.sleep(0.1)
+        w.stop()
+        t.join()
+
+        a_calls = [c for c in capture if c == 'a']
+        self.assertGreater(0, len(a_calls))
+
+    def test_period_double_no_immediate(self):
+        capture = []
+
+        @periodic.periodic(0.01, run_immediately=False)
+        def a():
+            capture.append('a')
+
+        @periodic.periodic(0.02, run_immediately=False)
+        def b():
+            capture.append('b')
+
+        w = periodic.PeriodicWorker([a, b])
+        t = tu.daemon_thread(target=w.start)
+        t.start()
+        time.sleep(0.1)
+        w.stop()
+        t.join()
+
+        b_calls = [c for c in capture if c == 'b']
+        self.assertGreater(0, len(b_calls))
+        a_calls = [c for c in capture if c == 'a']
+        self.assertGreater(0, len(a_calls))
+
+    def test_start_nothing_error(self):
+        w = periodic.PeriodicWorker([])
+        self.assertRaises(RuntimeError, w.start)
+
+    def test_missing_function_attrs(self):
+
+        def fake_periodic():
+            pass
+
+        cb = fake_periodic
+        self.assertRaises(ValueError, periodic.PeriodicWorker, [cb])
